﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Data;
using System.Windows.Interactivity;
using Infragistics.Windows.DockManager;

namespace XamDockManagerLayout
{
    public class RestoreDockManagerLayout : Behavior<XamDockManager>
    {
        #region Layout

        public string Layout
        {
            get { return (string)GetValue(LayoutProperty); }
            set { SetValue(LayoutProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Layout.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty LayoutProperty =
            DependencyProperty.Register("Layout", typeof(string), typeof(RestoreDockManagerLayout), new PropertyMetadata(string.Empty, OnLayoutChanged));

        private static void OnLayoutChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            // make sure this code only executes when the view model is trying to restore the layout
            RestoreDockManagerLayout behavior = d as RestoreDockManagerLayout;
            if (!behavior._saving)
            {
                string layout = (string)e.NewValue;
                if (behavior.AssociatedObject != null && !string.IsNullOrEmpty(layout))
                    behavior.AssociatedObject.LoadLayout(layout);
            }
        }

        #endregion

        #region RaisePinEvents

        /// <summary>
        /// RaisePinEvents Attached Dependency Property
        /// </summary>
        public static readonly DependencyProperty RaisePinEventsProperty =
            DependencyProperty.RegisterAttached("RaisePinEvents", typeof(bool), typeof(RestoreDockManagerLayout),
                new FrameworkPropertyMetadata(false, new PropertyChangedCallback(OnRaisePinEventsChanged)));

        private static void OnRaisePinEventsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ContentPane pane = d as ContentPane;

            if (null != pane)
            {
                if (true.Equals(e.NewValue))
                    pane.SetBinding(RestoreDockManagerLayout.IsPinnedValueProperty, Infragistics.Windows.Utilities.CreateBindingObject(ContentPane.IsPinnedProperty, BindingMode.OneWay, pane));
                else
                    pane.ClearValue(RestoreDockManagerLayout.IsPinnedValueProperty);
            }
        }

        /// <summary>
        /// Determines if the <see cref="Pinned"/> and <see cref="Unpinned"/> events should be raised for the pane.
        /// </summary>
        /// <param name="d">The pane on which to get the value</param>
        [AttachedPropertyBrowsableForType(typeof(ContentPane))]
        public static bool GetRaisePinEvents(DependencyObject d)
        {
            return (bool)d.GetValue(RaisePinEventsProperty);
        }

        /// <summary>
        /// Indicates if the <see cref="Pinned"/> and <see cref="Unpinned"/> events should be raised for the pane.
        /// </summary>
        /// <param name="d">The pane whose is pinned state is to be monitored</param>
        /// <param name="value">A boolean indicating whether to watch the events</param>
        public static void SetRaisePinEvents(DependencyObject d, bool value)
        {
            d.SetValue(RaisePinEventsProperty, value);
        }

        #endregion // RaisePinEvents

        #region IsPinnedValue

        private static readonly DependencyProperty IsPinnedValueProperty =
            DependencyProperty.RegisterAttached("IsPinnedValue", typeof(bool?), typeof(RestoreDockManagerLayout),
                new FrameworkPropertyMetadata(null, new PropertyChangedCallback(OnIsPinnedValueChanged)));

        private static void OnIsPinnedValueChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ContentPane pane = d as ContentPane;

            // the value is null when being initialized so don't raise the event
            if (e.OldValue != null && null != pane)
            {
                RoutedEventArgs args = new RoutedEventArgs(pane.IsPinned ? PinnedEvent : UnpinnedEvent);
                pane.RaiseEvent(args);
            }
        }

        #endregion //IsPinnedValue

        #region Pinned

        public static readonly RoutedEvent PinnedEvent = EventManager.RegisterRoutedEvent("Pinned",
            RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(RestoreDockManagerLayout));

        public static void AddPinnedHandler(UIElement element, RoutedEventHandler handler)
        {
            element.AddHandler(PinnedEvent, handler);
        }

        public static void RemovePinnedHandler(UIElement element, RoutedEventHandler handler)
        {
            element.RemoveHandler(PinnedEvent, handler);
        }
        #endregion // Pinned

        #region Unpinned

        public static readonly RoutedEvent UnpinnedEvent = EventManager.RegisterRoutedEvent("Unpinned",
            RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(RestoreDockManagerLayout));

        public static void AddUnpinnedHandler(UIElement element, RoutedEventHandler handler)
        {
            element.AddHandler(UnpinnedEvent, handler);
        }

        public static void RemoveUnpinnedHandler(UIElement element, RoutedEventHandler handler)
        {
            element.RemoveHandler(UnpinnedEvent, handler);
        }
        #endregion // Unpinned
        
        protected override void OnAttached()
        {
            base.OnAttached();

            AssociatedObject.Loaded += AssociatedObject_Loaded;
            AssociatedObject.PaneDragEnded += AssociatedObject_PaneDragEnded;
            AssociatedObject.Panes.CollectionChanged += Panes_CollectionChanged;
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();

            AssociatedObject.Loaded -= AssociatedObject_Loaded;
            AssociatedObject.PaneDragEnded -= AssociatedObject_PaneDragEnded;
            AssociatedObject.Panes.CollectionChanged -= Panes_CollectionChanged;
        }

        bool _isLoading = false;
        void AssociatedObject_Loaded(object sender, RoutedEventArgs e)
        {
            // when the dockmanager is restored, the layout needs to be restored as well
            if (!string.IsNullOrEmpty(Layout))
            {
                _isLoading = true;
                AssociatedObject.LoadLayout(Layout);
                _isLoading = false;
            }
        }

        void AssociatedObject_PaneDragEnded(object sender, Infragistics.Windows.DockManager.Events.PaneDragEndedEventArgs e)
        {
            SaveLayout();
        }

        void Panes_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            IEnumerable<ContentPane> panes = AssociatedObject.GetPanes(PaneNavigationOrder.VisibleOrder);
            foreach (ContentPane pane in panes)
            {
                SetRaisePinEvents(pane, true);
                pane.Loaded += pane_Loaded;
                pane.Unloaded += pane_Unloaded;
            }
        }

        void pane_Loaded(object sender, RoutedEventArgs e)
        {
            AddPinnedHandler(sender as ContentPane, OnPanePinned);
            AddUnpinnedHandler(sender as ContentPane, OnPaneUnpinned);
        }

        void pane_Unloaded(object sender, RoutedEventArgs e)
        {
            RemovePinnedHandler(sender as ContentPane, OnPanePinned);
            RemoveUnpinnedHandler(sender as ContentPane, OnPaneUnpinned);
        }

        private void OnPaneUnpinned(object sender, RoutedEventArgs e)
        {
            SaveLayout();
        }

        private void OnPanePinned(object sender, RoutedEventArgs e)
        {
            SaveLayout();
        }

        bool _saving = false;
        private void SaveLayout()
        {
            if (!_isLoading)
            {
                // this saves the layout to the viewmodel so it can be restored later
                _saving = true;
                Layout = AssociatedObject.SaveLayout();
                _saving = false;
            }
        }
    }
}
